#!/usr/bin/python3
# coding: utf-8

#       CopyRight 2020 Allan Psicobyte (psicobyte@gmail.com)
#
#       This program is free software: you can redistribute it and/or
#       modify it  under the terms of the GNU Affero General Public
#       License as published by the Free Software Foundation, either
#       version 3 of the License, or (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""
This module is the Pi Mondrian command line
"""

import argparse
import sys
import pimondrian as pm

DEFAULT_PAINT_NAME = "PiMondrian"
DEFAULT_LINE_SIZE = 4
DEFAULT_ITERATIONS = 5
DEFAULT_SIZE = (1200, 800)
DEFAULT_COLORS_FILE = pm.COLORS_FILE
DEFAULT_NUMBERS_FILE = pm.PI_DIGITS_FILE


def main():
    """
    CLI function
    """
    help_txt = """
Program that draws pictures in the style of Piet Mondrian.
This program creates a PNG file with a painting from the digits of the
umber Pi. Optionally, you can indicate a file with numbers or send the
numbers through a pipe to the standard input.
"""
    epilog_txt = """
This program works recursively, first dividing the canvas into two
rectangles, then dividing each of them into two others, and so on, as many
times as indicated with the flag --iterations.
"""

    parser = argparse.ArgumentParser(prog="pimondrian",
                                     description=help_txt,
                                     epilog=epilog_txt)
    parser.add_argument('-f', '--file', type=str, dest='file',
                        help='path of the file containing the numbers. If not \
                        specified, the first 10000 digits of Pi will be used')

    parser.add_argument('-c', '--colors', type=str, dest='colors',
                        help='path of the file containing the colors')

    parser.add_argument('-n', '--name', type=str, dest='name',
                        help='painting name. The default value is {}'.format(
                            DEFAULT_PAINT_NAME))

    parser.add_argument('-g', '--gallery', type=int, dest='gallery',
                        help='number of generated paintings')

    parser.add_argument('-y', '--ysize', type=int, dest='y_size',
                        help='painting height. The default value is {}'.format(
                            DEFAULT_SIZE[1]))

    parser.add_argument('-x', '--xsize', type=int, dest='x_size',
                        help='painting width. The default value is {}'.format(
                            DEFAULT_SIZE[0]))

    parser.add_argument('-i', '--iterations', type=int, dest='iterations',
                        help='number of iterations in the creation of the \
                        painting. The more iterations, the more rectangles. \
                        The default value is {}'.format(DEFAULT_ITERATIONS))

    parser.add_argument('-l', '--line', type=int, dest='line',
                        help='line thickness. The default value is {}'.format(
                            DEFAULT_LINE_SIZE))

    parser.add_argument('-r', '--random', action='count',
                        help='use random numbers instead of a file')

    args = parser.parse_args()

    paint_name = args.name if args.name else DEFAULT_PAINT_NAME

    size = (args.x_size if args.x_size else DEFAULT_SIZE[0],
            args.y_size if args.y_size else DEFAULT_SIZE[1])

    iterations = args.iterations if args.iterations else DEFAULT_ITERATIONS

    line = args.line if args.line else DEFAULT_LINE_SIZE

    c_filename = args.colors if args.colors else DEFAULT_COLORS_FILE
    try:
        colors_file = open(c_filename)
    except (FileNotFoundError, PermissionError, IsADirectoryError):
        sys.exit("error opening colors file")

    colors = pm.from_file_colors(colors_file)

    if not sys.stdin.isatty():
        print("Reading from stdin")
        generator = pm.from_file_generator(sys.stdin)
    elif args.random and not args.file:
        generator = pm.random_generator()
        print("Randomly generated")
    else:
        n_filename = args.file if args.file else DEFAULT_NUMBERS_FILE
        try:
            file = open(n_filename)
        except (FileNotFoundError, PermissionError, IsADirectoryError):
            sys.exit("error opening numbers file")
        generator = pm.from_file_generator(file)
        print("Extracting digits from file")

    if args.gallery:
        for number in range(args.gallery):
            print("Artist working... ({})".format(number))
            pm.Painting(iterations, generator).save_png(line,
                                                        colors,
                                                        size,
                                                        "{}_{}".format(
                                                            paint_name,
                                                            number))
    else:
        print("Artist working...")
        pm.Painting(iterations, generator).save_png(line,
                                                    colors,
                                                    size,
                                                    paint_name)


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# coding: utf-8

#       CopyRight 2020 Allan Psicobyte (psicobyte@gmail.com)
#
#       This program is free software: you can redistribute it and/or
#       modify it  under the terms of the GNU Affero General Public
#       License as published by the Free Software Foundation, either
#       version 3 of the License, or (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Module that generates paintings in the Mondrian style from a numerical
sequence.
"""
import os
from random import randint
import sys
from PIL import Image, ImageDraw

COLORS_FILE = os.path.join(os.path.dirname(__file__), 'colors.txt')
PI_DIGITS_FILE = os.path.join(os.path.dirname(__file__), '10000pi.txt')


def from_file_generator(file):
    """
    Return a generator that yield the characters in file if they are numbers.
    For each character that is not a number, returns zero instead.
    @param file:  file object
    @return: generator
    """
    while 1:
        character = file.read(1)
        if not character:
            return
        try:
            character = int(character)
        except ValueError:
            character = 0
        yield character


def pi_generator():
    """
    Return a generator that yield the digits of number Pi.
    @return: generator
    """
    try:
        pi_file = open(PI_DIGITS_FILE)
    except (FileNotFoundError, PermissionError, IsADirectoryError):
        sys.exit("error opening pi digits file")
    return from_file_generator(pi_file)


def random_generator():
    """
    Return a generator that yield random digits.
    @return: generator
    """
    while 1:
        yield randint(0, 9)


def from_file_colors(file):
    """
    The expected file format consists of ten lines, each containing a color
    in the form #RRGGBB
    @param file: file object
    @return: list
    """
    list_colors = file.readlines()
    return list_colors[:10]


def default_colors():
    """
    Return a a list with colors.
    @return: list
    """
    try:
        color_file = open(COLORS_FILE)
    except (FileNotFoundError, PermissionError, IsADirectoryError):
        sys.exit("error opening colors file")
    return from_file_colors(color_file)


class Painting:
    """
    A painting like those that Mondrian painted
    """

    def __init__(self, iterations, iterator):
        """
        @param iterations: number of iterations
        @param iterator: numbers iterator
        """
        self._max_gen = iterations
        self._iterator = iterator

        # root rectangle
        self._rectangles = [SubRectangle(1, 0, 0)]

        # rest of rectangles
        i = 0
        while i < iterations:
            for rectangle in self._rectangles:
                if rectangle.generation == i:
                    self._divide_rectangle(rectangle)
            i += 1

    def _get_generation(self, gen=None):
        """
        Returns the rectangles of the iteration indicated in gen. If gen is
        not provided, the last iteration is returned.
        @param gen: Generation
        @return: tuple of rectangles
        """
        if gen:
            return (x for x in self._rectangles if x.generation == gen)
        return (x for x in self._rectangles if x.generation == self._max_gen)

    def _divide_rectangle(self, parent):
        """Take the next three digits of the iterator, and divide the
        rectangle parent in two using the first of those three digits as a
        proportion. The other two digits determine the color of the resulting
        rectangles.
        @param parent: parent rectangle"""

        (cut_point, color1, color2) = (next(self._iterator, 0),
                                       next(self._iterator, 0),
                                       next(self._iterator, 0))

        self._rectangles.extend(
            (SubRectangle(0, cut_point, color1, parent),
             SubRectangle(1, cut_point, color2, parent))
        )

    def draw(self, line, colors, size, gen=None):
        """
        Draws a paint
        @param line: Line width.
        @param colors: List with the color table
        @param size: tuple (x,y) with de size in pixels
        @param gen: The iteration number of which the rectangles will be used.
        If not provided, the last iteration will be used.
        @return: PIL image object
        """

        img = Image.new("RGB", size, "#000000")
        draw = ImageDraw.Draw(img)
        line_w = round(line / 2, 0)

        for rect in self._get_generation(gen):
            zero = tuple(
                z1 * z2 / 100 + line_w for z1, z2 in zip(rect.zero, size))
            end = tuple(
                e1 * e2 / 100 - line_w for e1, e2 in zip(rect.end, size))

            draw.rectangle([zero, end], outline="#000000",
                           fill=colors[rect.color])
        return img

    def save_png(self, line, colors, size, name, gen=None):
        """
        Save PNG image
        @param name: File name. The PNG extension will be added.
        @param line: Line width.
        @param colors: List with the color table.
        @param size: tuple (x,y) with de size in pixels.
        @param gen: The iteration number of which the rectangles will be used.
        If not provided, the last iteration will be used.
        """
        img = self.draw(line, colors, size, gen)
        img.save(name + '.png', "PNG")


class SubRectangle:
    """
    Colored rectangle as a subdivision of a parent rectangle.
    """

    def __init__(self, position, cut_point, color, parent=None):
        """
        Create a rectangle with coordinates and color as a subdivision of a
        parent rectangle.
        @param position: 0 for the first rectangle. 1 for the second rectangle.
        @param cut_point: 0-10 number. Proportional position of the
        dividing line.
        @param color: color number 0-10.
        @param parent: parent rectangle Id or None.
        """
        self.parent = parent
        self.color = color

        if not parent:
            self._zero_x = 0
            self._zero_y = 0
            self._end_x = 100.0
            self._end_y = 100.0
            self.vert = True
            self.generation = 0

        else:
            self.generation = parent.generation + 1
            self.vert = not parent.vert

            p_z = parent.zero
            p_e = parent.end

            if self.vert:
                self._zero_x = p_z[0]
                self._end_x = p_e[0]

                if position == 0:
                    self._zero_y = p_z[1]
                    self._end_y = self._divide(p_z[1], p_e[1], cut_point)
                else:
                    self._zero_y = self._divide(p_z[1], p_e[1], cut_point)
                    self._end_y = p_e[1]

            else:
                self._zero_y = p_z[1]
                self._end_y = p_e[1]

                if position == 0:
                    self._zero_x = p_z[0]
                    self._end_x = self._divide(p_z[0], p_e[0], cut_point)
                else:
                    self._zero_x = self._divide(p_z[0], p_e[0], cut_point)
                    self._end_x = p_e[0]

    @property
    def zero(self):
        """
        Percentage coordinates of the point of origin of the rectangle
        @return: tuple
        """
        return self._zero_x, self._zero_y

    @property
    def end(self):
        """
        Percentage coordinates of the point of end of the rectangle
        @return: tuple
        """
        return self._end_x, self._end_y

    @staticmethod
    def _divide(short, long, factor):
        """
        return the point that corresponds to 'factor' tenths between 'short'
        and 'long'
        """

        if factor:
            return round(short + ((long - short) * factor / 10), 0)
        return long



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pimondrian",
    version="1.1.0",
    author="Allan Psicobyte",
    author_email="psicobyte@gmail.com",
    description="Draw Paintings as Mondrian",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/psicobyte/pimondrian_module",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3 or "
        "later (AGPLv3+)",
        "Operating System :: OS Independent",
        "Topic :: Artistic Software",
    ],
    keywords=['mondrian', 'art', 'painting'],
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": ["pimondrian = pimondrian.__main__:main"]
    },
    package_data={
        'pimondrian': ['colors.txt', '10000pi.txt'],
    },
    install_requires=['pillow'],
    python_requires='>=3.6'
)
